
// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

//
// makematrix_frank --
//   Returns the Frank matrix of order n.  It is upper
//   Hessenberg with determinant 1.  
//   The parameter k = 0 is the default; if k = 1 the
//   elements are reflected about the anti-diagonal (1,n)--(n,1).
//
//   The Frank matrix has all positive eigenvalues and they occur in reciprocal pairs
//   (so that 1 is an eigenvalue if n is odd).
//   The eigenvalues of the Frank matrix may be obtained in terms of the zeros of the
//   Hermite polynomials.
//   The floor(n/2) smallest eigenvalues of the Frank matrix are ill conditioned,
//   the more so for bigger n.

//   det(makematrix_frank ( n )') comes out far from 1 for large n---see Frank (1958)
//   and Wilkinson (1960) for discussions.
//
//   This version incorporates improvements suggested by W. Kahan.
//
// Original author : Nicolas Higham
// Scilab port : Michael Baudin
//
// Calling sequences
//   m = makematrix_frank ( n )
//   m = makematrix_frank ( n , k )
// References
//   W.L. Frank, Computing eigenvalues of complex matrices by determinant
//     evaluation and by methods of Danilewski and Wielandt, J. Soc.
//     Indust. Appl. Math., 6 (1958), pp. 378-392 (see pp. 385, 388).
//
//   Nicolas Higham, Accuracy and Stability of Numerical Algorithms
//     A Gallery of Test Matrices
//
//   G.H. Golub and J.H. Wilkinson, Ill-conditioned eigensystems and the
//      computation of the Jordan canonical form, SIAM Review, 18 (1976),
//      pp. 578-619 (Section 13).
//   H. Rutishauser, On test matrices, Programmation en Mathematiques
//      Numeriques, Editions Centre Nat. Recherche Sci., Paris, 165,
//      1966, pp. 349-365.  Section 9.
//   J.H. Wilkinson, Error analysis of floating-point computation,
//      Numer. Math., 2 (1960), pp. 319-340 (Section 8).
//   J.H. Wilkinson, The Algebraic Eigenvalue Problem, Oxford University
//      Press, 1965 (pp. 92-93).
//   The next two references give details of the eigensystem, as does
//   Rutishauser (see above).
//   P.J. Eberlein, A note on the matrices denoted by B_n, SIAM J. Appl.
//      Math., 20 (1971), pp. 87-92.
//   J.M. Varah, A generalization of the Frank matrix, SIAM J. Sci. Stat.
//      Comput., 7 (1986), pp. 835-839.
//
function m = makematrix_frank ( varargin )
  [lhs,rhs]=argn();
  if ( rhs < 1 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while at least 1 is expected."), "makematrix_frank", rhs);
    error(errmsg)
  end
  n = varargin(1)
  if ( rhs < 2 ) then
    k = 0
  else
    k = varargin(2)
  end
  p = n:-1:1
  m = triu( p( ones(n,1), :) - diag( ones(n-1,1), -1), -1 )
  // m = m(p,p)' reverses all the rows and columns of m
  if ( k <> 0 ) then
    m = m(p,p)'
  end
endfunction

